
/*
*===========================================================
*        File Name: channel.h
*          Version: A1
*          Created: 2015-8-14
*         Revision: 00
*         Compiler: ARMCC.exe V5.03.0.76
*           Author: 
*          Company: Analog Devices Inc
*===========================================================
*/
#ifndef CH0352_H_
#define CH0352_H_

#define ENABLE_CAL_FLOAT_RESULT


#include "inc\ADuCM360.h"
#include "inc\system_ADUCM360.h"
#include "i2c.h"
#include "adc.h"
#include "adp5065.h"
#include "adg715.h"
#include "battery.h"
#include "channel.h"
#include "task.h"
#include "flash.h"
#include "cmd.h"
#include "Thermistor10K_ERTJZEG103JA.h"


//Define the Address for Data Stored in Flash.
#define UserFlashPage 0x0001FC00
#define ADDR_MODULE_INDEX ((unsigned long *)UserFlashPage)
	
#define MIN_TASK_CYCLE 5


typedef	struct{					//ValidCH_T
	unsigned char qty;
	union{			//flag
		unsigned char val;
		struct{			//bits
			unsigned char CH0		: 1;
			unsigned char CH1		: 1;
			unsigned char CH2		: 1;
			unsigned char CH3		: 1;
			unsigned char CH4		: 1;
			unsigned char CH5		: 1;
			unsigned char CH6		: 1;
			unsigned char CH7		: 1;
		}bits;
	}flag;
	unsigned char index[8];
}ValidCH_T;
typedef struct{					//OPERATION_T
	struct {								//cfg
		//Initialization
		void (*init)(void);
		void (*adp5065_init)(void);
		void (*baudrateupd)(void);
		unsigned char (*moduleIDupd)(void);
		//Charging Parameter Configuration
		void (*InCurrLimit)(CHANNEL_T *ptr, unsigned char data);
		void (*TrickleVth)(CHANNEL_T *ptr, unsigned char data);
		void (*TrickleCurr)(CHANNEL_T *ptr, unsigned char data);
		void (*FastCurr)(CHANNEL_T *ptr, unsigned char data);
		void (*TermVolt)(CHANNEL_T *ptr, unsigned char data);
		void (*TermCurr)(CHANNEL_T *ptr, unsigned char data);
		void (*DisTermVolt)(CHANNEL_T *ptr, unsigned char data[3]);
		void (*DisTermCurr)(CHANNEL_T *ptr, unsigned char data[3]);
		void (*CHGStart)(CHANNEL_T *ptr);
		void (*CHGStop)(CHANNEL_T *ptr);
		void (*DHGStart)(CHANNEL_T *ptr);
		void (*DHGStop)(CHANNEL_T *ptr);
		void (*sw_reset)(void);
		void (*test_cycle)(unsigned short data);
		void (*CHG_CFG_ALL)(FORMAT_CFG_ALL_T* cfg);
		void (*InCurrLimit_Allset)(unsigned char data);
		void (*TrickleVth_Allset)(unsigned char data);
		void (*TrickleCurr_Allset)(unsigned char data);
		void (*FastCurr_Allset)(unsigned char data);
		void (*TermVolt_Allset)(unsigned char data);
		void (*TermCurr_Allset)(unsigned char data);
		void (*DisTermVolt_Allset)(unsigned char data[3]);
		void (*DisTermCurr_Allset)(unsigned char data[3]);
		void (*CHGStart_Allset)(void);
		void (*CHGStop_Allset)(void);
		void (*adp5065_falut_recover)(void);
		float (*fan_ctrl)(void);
		
		void (*start_volt_curr)(void);
		void (*start_temp_vdac)(void);
		void (*start_temp_sync)(void);
		void (*Vdisc_Refresh)(void);
		void (*Adp5065_RB)(void);
		void (*Adp5065_WR)(void);
	}cfg;
	struct {
		void(*start_ch)(CHANNEL_T *ptr);
		void (*start_all)(void);
		void(*stop_ch)(CHANNEL_T *ptr);
		void(*stop_all)(void);
	}test;
	struct {								//info
		//Valid Channel Scan
		void (*validchsacn)(void);
		void (*volt)(BATTERY_T *ptr);
		void (*curr)(BATTERY_T *ptr);
		void (*temp)(BATTERY_T *ptr);
		void (*temp_sync)(BATTERY_T *ptr);
		void (*vdac)(BATTERY_T *ptr);
		void (*status)(CHANNEL_T *ptr);
		void (*monitor)(void);
	}rb;
} OPERATION_T;
typedef struct{					//CN0352_T
	const float dchg_term_volt[23];
	const float dchg_term_curr[21];
//	const unsigned short dchg_term_raw_volt[23];
	const unsigned short dchg_term_raw_curr[21];
	const uint16_t name;	//CN0352
	const uint16_t FirmVer;			//A001
	uint8_t  ModuleID;		//There would be Multi-Module on RS485 Bus 1~255;	
	ValidCH_T validch;
	unsigned char adc_odr_index;
	float samplerate;
	float tsettle;			//settling time
	unsigned int baudrate;
	unsigned char CHNO_BAT_Vdis;			//Store the Channel Number for Discharging Voltage Refreshing
	unsigned char CHNO_BAT_Value;				//Store the Channel Number for Battery Value Read back
	unsigned char CHNO_ADP5065_RB;		//Store the Channel Number for ADP5065 Status Write and Read back.
	unsigned char CHNO_ADP5065_WR;		//Store the Channel Number for ADP5065 Status Write and Read back.	
	unsigned char INDEX_ADP5065_REG_RB;
	unsigned char INDEX_ADP5065_REG_WR;
	unsigned char test_start;
	unsigned short test_cycle;
	unsigned short test_cycle_cnt;
	CHANNEL_T ch[8];
	OPERATION_T run;
} CN0352_T;
extern CN0352_T cn0352;



//typedef struct {
//	uint16_t * pname;
//	uint16_t * FirmVer;
//	uint8_t * pModuleID;
//	ValidCH_T* pvalidch;
//}


//typedef struct {
//	unsigned char *p[2+2+1+sizeof(ValidCH_T)];			//name + FirmVer + ModuleID + ValidCH_T
//	unsigned char length;
//} CN0352_MODU_INFO_T;

//extern CN0352_MODU_INFO_T cn0352_module_info;


#endif /*CH0352_H_*/

